@@ -1,12 +1,11 @@
 # -*- coding: utf-8 -*-
-import django
 from django.conf import settings
 from django.utils.http import urlquote
 from django.utils.encoding import force_str
 from django.shortcuts import redirect, resolve_url
 from django.contrib.auth import REDIRECT_FIELD_NAME
 
-from .helpers import get_redir_path, check_allow_impersonate, is_authenticated
+from .helpers import get_redir_path, check_allow_impersonate
 
 
 def get_login_url():
@@ -15,7 +14,7 @@ def get_login_url():
 
 def allowed_user_required(view_func):
     def _checkuser(request, *args, **kwargs):
-        if not is_authenticated(request.user):
+        if not request.user.is_authenticated:
             return redirect(u'{0}?{1}={2}'.format(
                 get_login_url(),
                 REDIRECT_FIELD_NAME,
