@@ -1,12 +1,9 @@
 # -*- coding: utf-8 -*-
-from .helpers import (
-    User, check_allow_for_user, check_allow_for_uri, is_authenticated,
-)
+from django.http import HttpResponseNotAllowed
+from django.utils.deprecation import MiddlewareMixin
 
-try:
-    from django.utils.deprecation import MiddlewareMixin
-except ImportError:
-    MiddlewareMixin = object  # fallback for Django < 1.10
+from .helpers import User, check_allow_for_uri, check_allow_for_user
+from .settings import settings
 
 
 class ImpersonateMiddleware(MiddlewareMixin):
@@ -14,8 +11,7 @@ class ImpersonateMiddleware(MiddlewareMixin):
         request.user.is_impersonate = False
         request.impersonator = None
 
-        if is_authenticated(request.user) and \
-           '_impersonate' in request.session:
+        if request.user.is_authenticated and '_impersonate' in request.session:
             new_user_id = request.session['_impersonate']
             if isinstance(new_user_id, User):
                 # Edge case for issue 15
@@ -26,8 +22,12 @@ class ImpersonateMiddleware(MiddlewareMixin):
             except User.DoesNotExist:
                 return
 
-            if check_allow_for_user(request, new_user) and \
-               check_allow_for_uri(request.path):
+            if settings.READ_ONLY and request.method not in ['GET', 'HEAD']:
+                return HttpResponseNotAllowed(['GET', 'HEAD'])
+
+            if check_allow_for_user(request, new_user) and check_allow_for_uri(
+                request.path
+            ):
                 request.impersonator = request.user
                 request.user = new_user
                 request.user.is_impersonate = True
